"""Test config loaders"""

import unittest
from dataclasses import dataclass, field

from aiobp.config.conf import loader as conf_loader
from aiobp.config.yaml import loader as json_loader
from aiobp.config.yaml import loader as yaml_loader


@dataclass
class Server:
    host: str = "10.20.30.40"
    port: int = 65432


@dataclass
class Client:
    enabled: bool = True
    names: list[str] = field(default_factory=list)
    ids: list[int] = field(default_factory=list)
    note: str = "Test"


@dataclass
class Config:
    server: Server = field(default_factory=Server)
    client: Client = field(default_factory=Client)
    users: dict[str, str] = field(default_factory=dict)


expected = Config(
    server=Server(
        host="127.0.0.1",
        port=12345,
    ),
    client=Client(
        enabled=False,
        names=["a", "b", "c"],
        ids=[1, 2, 3],
        note="Testing",
    ),
    users={
        "kenny": "Kenneth McCormick",
        "cartman": "Eric Cartman",
    },
)

defaults = Config(
    server=Server(
        host="10.20.30.40",
        port=65432,
    ),
    client=Client(
        enabled=True,
        names=[],
        ids=[],
        note="Test",
    ),
    users={},
)

class TestConfigLoaders(unittest.TestCase):
    def test_conf_loader(self) -> None:
        config = conf_loader(Config, "tests/config.conf")
        assert config == expected

    def test_yaml_loader(self) -> None:
        config = yaml_loader(Config, "tests/config.yaml")
        assert config == expected

    def test_json_loader(self) -> None:
        config = json_loader(Config, "tests/config.json")
        assert config == expected

    def test_conf_defaults(self) -> None:
        config = conf_loader(Config)
        assert config == defaults

    def test_json_defaults(self) -> None:
        config = json_loader(Config)
        assert config == defaults

    def test_yaml_defaults(self) -> None:
        config = yaml_loader(Config)
        assert config == defaults

if __name__ == "__main__":
    unittest.main()



import inspect
from typing import Any, Generic, TypeVar, get_args, get_origin

from aiohttp import web

T = TypeVar('T')

def dashed(s: str) -> list[str]:
    return s.split("-")

class Path(Generic[T]):

    @staticmethod
    def provide(request: web.Request, key: str, value_type: list[T]) -> Any:
        value = request.match_info.get(key)
        if value is None:
            return None

        if value_type:
            return value_type[0](value)

        return value

class Provider:

    def __init__(self):
        self.__types = {
            web.Request: lambda request, *args: request,
            Path: Path.provide,
        }

    def __provide(self, request: web.Request, arg_name: str, parameter: inspect.Parameter) -> Any:
        # annotation type... get_origin(list[str]) -> list, "or" is here because get_origin(str) -> None
        parameter_type = get_origin(parameter.annotation) or parameter.annotation
        # annotation parameter arguments get_args(tuple[int, str, bool]) -> (int, str, bool)
        parameter_args = get_args(parameter.annotation)
        provider = self.__types.get(parameter_type)
        if provider is None:
            raise Exception("Invalid annotation")

        return provider(request, arg_name, parameter_args)

    @web.middleware
    async def middleware(self, request: web.Request, handler) -> web.Response:
        sig = inspect.signature(handler)
        kwargs = {arg_name: self.__provide(request, arg_name, param) for arg_name, param in sig.parameters.items()}
        return await handler(**kwargs)



"""Load configuration from YAML file"""

from pathlib import Path
from typing import Annotated, Optional

import msgspec


def loader(config_class: type[Annotated], filename: Optional[str] = None) -> Annotated:
    """Load configuration from YAML file"""
    if filename is None:
        return msgspec.yaml.decode("{}", type=config_class)

    config = Path(filename).read_bytes()
    return msgspec.yaml.decode(config, type=config_class)




from aiobp import runner
from aiobp.aiohttp import Provider, Path, dashed
from aiohttp import web


async def hello(request: web.Request, who: Path[dashed]) -> web.Response:
    return web.Response(text=f"Hello, {who}")


provider = Provider()

async def main():
    app = web.Application(middlewares=[provider.middleware])
    app.add_routes([web.get('/{who}', hello)])
    app_runner = web.AppRunner(app)
    await app_runner.setup()
    site = web.TCPSite(app_runner, '0.0.0.0', 8080)
    await site.start()


if __name__ == "__main__":
    runner(main())